@@ -222,7 +222,7 @@ double vector_sd(std::vector<double> &vector1) {
     double vec_mean = vector_mean(vector1);
     double vec_sd = 0.0;
     for (double vector1_value:vector1) {
-        vec_sd += pow(vector1_value - vec_mean, 2);
+        vec_sd += pow(vector1_value - vec_mean, 2.0);
     }
     vec_sd = vec_sd / (vector1.size() - 1);
     vec_sd = sqrt(vec_sd);
@@ -306,7 +306,7 @@ std::vector<std::vector<double>> Euclidean_distance(std::vector<double> &matrix,
 double square_Euclidean_distance(std::vector<double> &vector1, std::vector<double> &vector2) {
     double distance_value = 0.0;
     for (uint i = 0; i < vector1.size(); ++i) {
-        distance_value += pow(vector1[i] - vector2[i], 2);
+        distance_value += pow(vector1[i] - vector2[i], 2.0);
     }
     return distance_value;
 }
@@ -315,7 +315,7 @@ double weight_square_Euclidean_distance(std::vector<double> &vector1, std::vecto
                                         std::vector<double> &weight) {
     double distance_value = 0.0;
     for (uint i = 0; i < vector1.size(); ++i) {
-        distance_value += weight[i] * pow(vector1[i] - vector2[i], 2);
+        distance_value += weight[i] * pow(vector1[i] - vector2[i], 2.0);
     }
     return distance_value;
 }
@@ -770,55 +770,6 @@ void quick_sort_dataset2(std::vector<std::tuple<int, double, double, double>> &d
     std::sort(dataset.begin(), dataset.end(), quick_sort_dataset2_compare);
 }
 
-double compute_condition_ball_covariance_crude(std::vector<std::vector<double>> &distance_x,
-                                               std::vector<std::vector<double>> &distance_y,
-                                               std::vector<std::vector<double>> &kernel_density_estimation) {
-    int num = (int) kernel_density_estimation.size();
-    std::vector<double> condition_ball_covariance(kernel_density_estimation.size(), 0.0);
-    double condition_ball_covariance_stats;
-
-    std::vector<double> kernel_rowsum(kernel_density_estimation.size(), 0.0);
-    for (int m = 0; m < num; ++m) {
-        kernel_rowsum[m] += vector_sum(kernel_density_estimation[m]);
-    }
-    std::vector<std::vector<double >> weight_delta_x_matrix(kernel_rowsum.size(),
-                                                            std::vector<double>(kernel_rowsum.size()));
-    std::vector<std::vector<double >> weight_delta_y_matrix(kernel_rowsum.size(),
-                                                            std::vector<double>(kernel_rowsum.size()));
-    std::vector<std::vector<double >> weight_delta_xy_matrix(kernel_rowsum.size(),
-                                                             std::vector<double>(kernel_rowsum.size()));
-    double delta_ijl_xy, delta_ijl_x, delta_ijl_y;
-    for (int l = 0; l < num; ++l) {
-        for (int i = 0; i < num; ++i) {
-            for (int j = 0; j < num; ++j) {
-                delta_ijl_xy = delta_ijl_x = delta_ijl_y = 0.0;
-                for (int k = 0; k < num; ++k) {
-                    delta_ijl_xy += distance_x[i][j] >= distance_x[i][k] && distance_y[i][j] >= distance_y[i][k]
-                                    ? kernel_density_estimation[k][l] : 0;
-                    delta_ijl_x += distance_x[i][j] >= distance_x[i][k] ? kernel_density_estimation[k][l] : 0;
-                    delta_ijl_y += distance_y[i][j] >= distance_y[i][k] ? kernel_density_estimation[k][l] : 0;
-                }
-                weight_delta_xy_matrix[i][j] = delta_ijl_xy;
-                weight_delta_x_matrix[i][j] = delta_ijl_x;
-                weight_delta_y_matrix[i][j] = delta_ijl_y;
-            }
-        }
-        for (int i = 0; i < num; ++i) {
-            for (int j = 0; j < num; ++j) {
-                condition_ball_covariance[l] +=
-                        (kernel_density_estimation[i][l] * kernel_density_estimation[j][l] / pow(kernel_rowsum[l], 2)) *
-                        (pow(weight_delta_xy_matrix[i][j] / kernel_rowsum[l] -
-                             weight_delta_x_matrix[i][j] * weight_delta_y_matrix[i][j] / pow(kernel_rowsum[l], 2), 2));
-            }
-        }
-        condition_ball_covariance[l] *= pow(kernel_rowsum[l], 6);
-        condition_ball_covariance[l] /= pow((double) num, 6.0);
-    }
-    condition_ball_covariance_stats = vector_mean(condition_ball_covariance);
-
-    return condition_ball_covariance_stats;
-}
-
 /**
  * V-type conditional distance covariance
  * @param distance_x : pairwise distance of variable x
@@ -847,7 +798,8 @@ double compute_condition_distance_covariance_crude(std::vector<std::vector<doubl
                         cross_dy3 = distance_y[i][l] + distance_y[k][j] - distance_y[i][k] - distance_y[j][l];
                         d_ijkl = cross_dx1 * cross_dy1 + cross_dx2 * cross_dy2 + cross_dx3 * cross_dy3;
 
-                        condition_distance_covariance[u] += d_ijkl * kernel_density_estimation[i][u] *
+                        condition_distance_covariance[u] += d_ijkl *
+                                                            kernel_density_estimation[i][u] *
                                                             kernel_density_estimation[j][u] *
                                                             kernel_density_estimation[k][u] *
                                                             kernel_density_estimation[l][u];
@@ -855,7 +807,7 @@ double compute_condition_distance_covariance_crude(std::vector<std::vector<doubl
                 }
             }
         }
-        condition_distance_covariance[u] /= pow((double) num, 4.0);
+        condition_distance_covariance[u] /= (num * num * num * num);
     }
     double condition_distance_covariance_stat = vector_mean(condition_distance_covariance);
     return condition_distance_covariance_stat;
